<?php

/**
 * Plugin Name:       Lynked Loyalty
 * Description:       A loyalty program allowing your customers collect rewards on their smart device online and in-store.
 * Version:           0.1.1
 * Requires at least: 5.3
 * Requires PHP:      7.0
 * Author:            Lynked Loyalty
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       lynked
 */

add_action( 'admin_menu', 'lynked_init_menu' );

/**
 * Init Admin Menu.
 *
 * @return void
 */
function lynked_init_menu(): void
{
    add_menu_page( __( 'Lynked', 'lynked'), __( 'Lynked', 'lynked'), 'manage_options', 'lynked', 'lynked_admin_page', plugins_url( 'lynked-loyalty/src/shared/img/menu_icon.png' ), '2.1' );
}

/**
 * Init Admin Page.
 *
 * @return void
 */
function lynked_admin_page(): void
{
    require_once plugin_dir_path( __FILE__ ) . 'templates/app.php';
}

add_action( 'admin_enqueue_scripts', 'lynked_admin_enqueue_scripts' );

function lynked_url_obj(): array
{
    return [
        'api_url' => 'https://api.lynked.ie',
        'dashboard_url' => 'https://app.lynked.ie'
    ];
}

/**
 * Enqueue scripts and styles.
 *
 * @return void
 */
function lynked_admin_enqueue_scripts(): void
{
    $urlArr = lynked_url_obj();
    wp_enqueue_style( 'lynked-admin-style', plugin_dir_url( __FILE__ ) . 'build/admin.css' );
    wp_register_script( 'lynked-admin-script', plugin_dir_url( __FILE__ ) . 'build/admin.js', array( 'wp-element' ), '1.0.0', true );
    $shop = parse_url(get_site_url())['host'];
    $args = array(
        'headers' => array(
            'Content-Type' => 'application/json'
        ),
        'body' => json_encode([
            'shop' => $shop
        ])
    );

    $response = wp_remote_post( sanitize_url($urlArr['api_url'].'/shopify/check-if-connected'), $args);
    $body     = wp_remote_retrieve_body( $response );
    $result = json_decode($body);
    $businessAccount = '';
    if($result->status){
        $businessAccount = $result->business_account;
    }
    $data = array(
        'shopConnected' => $result->status,
        'businessAccount' => $businessAccount,
        'apiUrl' => sanitize_url($urlArr['api_url']),
        'dashboardUrl' => sanitize_url($urlArr['dashboard_url'])
    );
    wp_localize_script( 'lynked-admin-script', 'plugin_args', $data );
    wp_enqueue_script( 'lynked-admin-script' );
}

function lynked_insert_html_in_header(): void
{
    $urlArr = lynked_url_obj();
    wp_enqueue_style( 'lynked-style', plugin_dir_url( __FILE__ ) . 'build/index.css' );
    wp_enqueue_script( 'lynked-script', plugin_dir_url( __FILE__ ) . 'build/index.js', array( 'wp-element' ), '1.0.0', true );
    wp_localize_script('lynked-script', 'wpApiSettings', [
        'nonce' => wp_create_nonce('wp_rest'),
        'apiUrl' => $urlArr['api_url'],
        'dashboardUrl' => $urlArr['dashboard_url']
    ]);
    echo '<div id="lynked-store"></div>';
}

/* Front End */
add_action( 'wp_head', 'lynked_insert_html_in_header' );

/**
 * This is our callback function that embeds our phrase in a WP_REST_Response
 */
function lynked_get_endpoint_phrase( $request )
{
    $urlArr = lynked_url_obj();

    $shop = sanitize_text_field($request['shop']);
    $email = sanitize_email($request['email']);
    $password = sanitize_text_field($request['password']);

    $store_name     = get_option( 'blogname' );
    $store_city        = get_option( 'woocommerce_store_city' );
    $store_raw_country = get_option( 'woocommerce_default_country' );
    $split_country = explode( ":", $store_raw_country );
    $store_country = $split_country[0];

    $addressShop = str_replace(" ", "+", $store_city . ' ' . $store_country);

    $responseCoordinate = wp_remote_get(sanitize_url('https://nominatim.openstreetmap.org/search?q=' . $addressShop . '&format=json'));
    $bodyCoordinate = json_decode(wp_remote_retrieve_body( $responseCoordinate ), true);
    $jsonToArr = reset($bodyCoordinate);
    $location = [
        'display_name' => $jsonToArr['display_name'],
        'lat' => $jsonToArr['lat'],
        'lon' => $jsonToArr['lon']
    ];

    $address = $location['display_name'];
    $lat = $location['lat'];
    $lon = $location['lon'];

    $args = array(
        'headers' => array(
            'Content-Type' => 'application/json'
        ),
        'body' => json_encode([
            'email' => $email,
            'password' => $password,
            'shop' => $shop,
            'address' => $address,
            'lat' => $lat,
            'lon' => $lon
        ])
    );
    $postUrl = sanitize_url($urlArr['api_url'].'/shopify/sign-in');
    $response = wp_remote_post( $postUrl, $args);
    $body     = wp_remote_retrieve_body( $response );

    return rest_ensure_response( [json_decode($body), $store_name ]);
}

add_action( 'rest_api_init', 'lynked_register_api_hooks' );

function lynked_register_api_hooks(): void
{
    register_rest_route(
        'lynked', '/login/',
        array(
            'methods' => WP_REST_Server::CREATABLE,
            'callback' => 'login',
            'permission_callback' => '__return_true',
        )
    );

    function login($request)
    {
        $login = (string) $request['login'];
        $password = (string) $request['password'];

        $data = array();
        $data['user_login'] = $request["login"];
        $data['user_password'] =  $request["password"];
        $data['remember'] = true;
        $user = wp_signon( $data, false );

        if ( !is_wp_error($user) ) {
            wp_set_auth_cookie($user->get('id'),true,is_ssl(),wp_get_session_token());
            return $user;
        }

        return [
            "login"=>$login,
            "password"=>$password
        ];
    }
}

/**
 * This function is where we register our routes for our example endpoint.
 */
function lynked_register_example_routes(): void
{
    // register_rest_route() handles more arguments, but we are going to stick to the basics for now.
    register_rest_route( 'lynked', '/connect-business', array(
        // By using this constant we ensure that when the WP_REST_Server changes our readable endpoints will work as intended.
        'methods'  => WP_REST_Server::CREATABLE,
        // Here we register our callback. The callback is fired when this endpoint is matched by the WP_REST_Server class.
        'callback' => 'lynked_get_endpoint_phrase',
        'permission_callback' => '__return_true',
    ) );
}

add_action( 'rest_api_init', 'lynked_register_example_routes' );

function lynked_get_shop_info_phrase() {
    $store_name     = get_option( 'blogname' );
    $currency = get_option('woocommerce_currency');
    $symbols = get_woocommerce_currency_symbol($currency);
    $user = wp_get_current_user();
    return rest_ensure_response( [
        'shop_name' => $store_name,
        'currency' => $currency,
        'currency_symbol'=>$symbols,
        'user' => $user
    ]);
}

add_action( 'rest_api_init', 'lynked_get_shop_info_routes' );

/**
 * This function is where we register our routes for our example endpoint.
 */
function lynked_get_shop_info_routes(): void
{
    register_rest_route( 'lynked', 'shop-info', array(
        'methods'  => WP_REST_Server::READABLE,
        'callback' => 'lynked_get_shop_info_phrase',
        'permission_callback' => '__return_true',
    ) );
}

function lynked_add_discount( $request ) {
    $code = sanitize_text_field($request['code']);
    $reward = absint($request['reward']);
    $is_add = isset( $request['isAdd'] ) && true === $request['isAdd'];
    if (!session_id()) {
        @session_start();
    }
    if (!isset($_SESSION)) { session_start();}
    if(wc_get_coupon_id_by_code($code)){
        $coupon = new WC_Coupon($code);
        if($is_add) {
            $_SESSION['coupon_code'] = $code;
        }
        return rest_ensure_response( [
            'coupon'=>$coupon->get_data()
        ]);
    }
    $coupon = new WC_Coupon();

    $coupon->set_code( $code );
    $coupon->set_amount( $reward );
    $coupon->set_individual_use(true);
    $coupon->set_usage_limit( 1 );
    $coupon->set_usage_limit_per_user( 1 );
    $coupon->save();

    if($is_add){
        $_SESSION['coupon_code'] = $code;
    }

    return rest_ensure_response( [
        'coupon'=>$coupon->get_data()
    ]);
}

add_action( 'rest_api_init', 'lynked_add_discount_routes' );

/**
 * This function is where we register our routes for our example endpoint.
 */
function lynked_add_discount_routes(): void
{
    register_rest_route( 'lynked', 'discount', array(
        'methods'  => WP_REST_Server::CREATABLE,
        'callback' => 'lynked_add_discount',
        'permission_callback' => '__return_true',
    ) );
}

add_action( 'woocommerce_order_status_completed', 'lynked_create_invoice_for_wc_order', 1);

function lynked_create_invoice_for_wc_order($order_id){
    $urlArr = lynked_url_obj();
    $order = wc_get_order($order_id);
    $coupons = $order->get_coupon_codes();
    $discountCode = '';
    if(count($coupons)){
        $discountCode = reset($coupons);
    }
    $args = array(
        'headers' => array(
            'Content-Type' => 'application/json'
        ),
        'body' => json_encode([
            'email' => $order->get_billing_email(),
            'id' => $order->get_id(),
            'total_price' => $order->get_total(),
            'currency' => $order->get_currency(),
            'referring_site' => $order->get_edit_order_url(),
            'discount_code' => $discountCode
        ])
    );

    $response = wp_remote_post( sanitize_url($urlArr['api_url'].'/shopify/webhook-checkout-creation'), $args);
    $body     = wp_remote_retrieve_body( $response );

    return rest_ensure_response( [json_decode($body), $discountCode ]);
}

add_action( 'user_register', 'lynked_registration_save', 10, 1 );

function lynked_registration_save( $user_id ) {
    $urlArr = lynked_url_obj();
    $user = get_user_by('id', $user_id);
    $args = array(
        'headers' => array(
            'Content-Type' => 'application/json'
        ),
        'body' => json_encode([
            'email' => $user->user_email,
            'first_name' => '',
            'last_name' => '',
        ])
    );

    $response = wp_remote_post( sanitize_url($urlArr['api_url'].'/shopify/webhook-customer-creation'), $args);
    $body     = wp_remote_retrieve_body( $response );

    error_log(json_encode($user), 0);
}

function lynked_apply_discount_to_cart() {
    if (!session_id()) {
        @session_start();
    }
    if(!isset($_SESSION['coupon_code'])){
        return;
    }
    $coupon_code = sanitize_text_field(esc_html($_SESSION['coupon_code']));
    if ( ! empty( $coupon_code ) && ! WC()->cart->has_discount( $coupon_code ) ){
        WC()->cart->add_discount( $coupon_code ); // apply the coupon discount
        unset($_SESSION['coupon_code']);
    }
}
add_action( 'woocommerce_before_cart', 'lynked_apply_discount_to_cart', 10, 0 );

/**
 * Auto Complete all WooCommerce orders.
 */
add_action( 'woocommerce_thankyou', 'lynked_woocommerce_auto_complete_order' );

function lynked_woocommerce_auto_complete_order( $order_id ) {
    if ( ! $order_id ) {
        return;
    }

    $order = wc_get_order( $order_id );
    $order->update_status( 'completed' );
}

function lynked_register_my_session()
{
    if( !session_id() )
    {
        session_start();
    }
}

add_action('init', 'lynked_register_my_session');